# -*- coding: utf-8 -*-
# Copyright 2016 by Avencall
# SPDX-License-Identifier: GPL-3.0+

from hamcrest import assert_that
from hamcrest import calling
from hamcrest import equal_to
from hamcrest import raises
from hamcrest import is_
from mock import Mock
import mock
from mock import sentinel
from unittest import TestCase
import json

from ari.exceptions import ARINotFound

from ..transfer import Transfer, TransferRole
from ..state import TransferStateAnswered, TransferStateEnded, TransferState, TransferStateEndedNotAnswered


class TestState(TestCase):

    def setUp(self):
        self.transfer_bridge = mock.MagicMock()
        type(self.transfer_bridge).content = mock.PropertyMock(return_value={
            'api'
            :
                '111'
            ,
            'client':
                '111'
            ,
            'event_reg':
                '111'
            ,
            'id':
                u'3a950430-9342-4ebd-9254-ba1a5bbd8fb5'
            ,
            'json':
                {
                    u'bridge_class'
                    :
                        u'stasis'
                    ,
                    u'bridge_type':
                        u'mixing'
                    ,
                    u'channels':
                        [
                            u'1480096473.7'
                        ]
                    ,
                    u'creator':
                        u'Stasis'
                    ,
                    u'id':
                        u'3a950430-9342-4ebd-9254-ba1a5bbd8fb5'
                    ,
                    u'name':
                        u'transfer'
                    ,
                    u'technology':
                        u'simple_bridge'
                }
        })

    def test_recipient_hangup_beep(self):
        ari = Mock()
        amid = Mock()
        notifier = Mock()
        services = Mock()
        state_persistor = Mock()
        transfer = Mock()
        a = TransferStateAnswered(amid, ari, notifier, services, state_persistor, transfer)
        a.recipient_hangup()
        assert_that(ari.channels.play.called, is_(True))

    def test_recipient_answer_transferred_call_abandoned(self):
        ari = Mock()
        amid = Mock()
        notifier = Mock()
        services = Mock()
        state_persistor = Mock()
        transfer = Mock()
        a = TransferStateAnswered(amid, ari, notifier, services, state_persistor, transfer)
        a.transferred_hangup()
        b = TransferStateEndedNotAnswered(amid, ari, notifier, services, state_persistor, transfer)
        b.recipient_answer_transferred_call_abandoned(self.transfer_bridge)

    def test_recipient_answer_transferred_call_abandoned_empty_bridge(self):
        ari = Mock()
        amid = Mock()
        notifier = Mock()
        services = Mock()
        state_persistor = Mock()
        transfer = Mock()
        transfer_bridge = None
        b = TransferStateEndedNotAnswered(amid, ari, notifier, services, state_persistor, transfer)
        self.assertRaises(AttributeError, b.recipient_answer_transferred_call_abandoned(transfer_bridge))

    def test_update_cache(self):
        ari = Mock()
        amid = Mock()
        notifier = Mock()
        services = Mock()
        state_persistor = Mock()
        transfer = Mock()
        b = TransferStateEnded(amid, ari, notifier, services, state_persistor, transfer)
        b.update_cache()
